@@ -8,7 +8,7 @@ from datetime import datetime, timezone, timedelta
 from myoperator.apm.decorators import collect_external_io_metrics, collect_function_duration_metrics
 from myoperator.apm.metrics import create_external_io_duration_metric_name
 from myoperator.apm.metrics import create_function_duration_metric_name
-from myoperator.apm.metrics import config_client
+from myoperator.apm.metrics import config_client, create_external_service_duration_metrics
 from myoperator.apm.metrics import create_function_duration_metrics, create_http_request_metrics
 
 PROMETHEUS_API_URL = os.environ.get('PROMETHEUS_API_URL', 'http://localhost:9090/api/v1/query')
@@ -23,9 +23,9 @@ class TestApmClientFunctions(unittest.TestCase):
 
     def test_create_function_duration_metrics(self):
         status_code = '2xx'
-        create_function_duration_metrics(status_code, METRIC_PREFIX)
+        create_function_duration_metrics(200, METRIC_PREFIX, 'TEST_FUNCTION', 'test_host', 'success')
         # invalid input
-        self.assertRaises(ValueError, create_function_duration_metrics, 200, METRIC_PREFIX)
+        self.assertRaises(ValueError, create_function_duration_metrics, 200, METRIC_PREFIX, 'TEST_FUNCTION', 'test_host', 'wrong_status')
 
     def test_create_http_request_metrics(self):
         status_code = '2xx'
@@ -35,6 +35,20 @@ class TestApmClientFunctions(unittest.TestCase):
         exception_type = type(ValueError('Test Error')).__name__
         create_http_request_metrics('5xx', duration_ms, METRIC_PREFIX, exception_type)
 
+        # wrong status
+        self.assertRaises(ValueError, create_http_request_metrics, 200, duration_ms, METRIC_PREFIX,)
+
+    def test_create_external_service_duration_metric(self):
+        status = 'error'
+        exception_class = type(RuntimeError('Test Runtime Error')).__name__
+        create_external_service_duration_metrics(status, 100, METRIC_PREFIX, 'test_service', 'test_service_address', exception_class)
+        create_external_service_duration_metrics('success', 100, METRIC_PREFIX, 'test_service', 'test_service_address')
+
+        # no exception class provided
+        self.assertRaises(ValueError, create_external_service_duration_metrics, status, 100, METRIC_PREFIX, 'test_service', 'test_service_address')
+        self.assertRaises(ValueError, create_external_service_duration_metrics, 'wrong_status', 100, METRIC_PREFIX, 'test_service', 'test_service_address')
+
+
 class TestConfigClient(unittest.TestCase):
 
     def test_config_client(self):
@@ -48,6 +62,45 @@ class TestConfigClient(unittest.TestCase):
         from myoperator.apm.metrics import statsd_client
         self.assertTrue(isinstance(statsd_client, statsd.TCPStatsClient))
 
+class TestDecorators(unittest.TestCase):
+
+    def test_collect_external_io_metrics(self):
+        class TestException(Exception):
+            pass
+
+        @collect_external_io_metrics(METRIC_PREFIX, service_name='test_service')
+        def dummy_func():
+            raise TestException('Error')
+
+        try:
+            dummy_func()
+        except TestException:
+            pass
+
+        @collect_external_io_metrics(METRIC_PREFIX, service_name='test_service')
+        def dummy_func():
+            pass
+
+        dummy_func()
+
+    def test_collect_function_metrics_decorator(self):
+        class TestException(Exception):
+            pass
+
+        @collect_function_duration_metrics(METRIC_PREFIX, function_name='test_dummy_func')
+        def dummy_func():
+            raise TestException('Error')
+
+        try:
+            dummy_func()
+        except TestException:
+            pass
+
+        @collect_function_duration_metrics(METRIC_PREFIX, function_name='test_dummy_func')
+        def dummy_func():
+            pass
+
+        dummy_func()
 
 class TestIntegrationDecorators(unittest.TestCase):
 
@@ -55,11 +108,10 @@ class TestIntegrationDecorators(unittest.TestCase):
         pass
 
     def tearDown(self):
+        config_client('TCP')
         pass
 
-
     def test_collect_external_io_metrics_decrator(self):
-
         @collect_external_io_metrics(METRIC_PREFIX, service_name='test_service')
         def dummy_func():
             choice = [True, False]
